
#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/xpm.h>
#include <X11/Xos.h>
#include "oximtool.h"
#include "gui.h"
#include "oxim.h"

#include "images/en.xpm"
#include "images/ch.xpm"
int check_tray_manager(gui_t *gui);

static int  tray_init = False;
static char **icon = NULL;

#define SYSTEM_TRAY_REQUEST_DOCK    0
#define SYSTEM_TRAY_BEGIN_MESSAGE   1
#define SYSTEM_TRAY_CANCEL_MESSAGE  2

static void send_message(
     Display* dpy, /* display */
     Window win,     /* sender (tray icon window) */
     long message, /* message opcode */
     long data1,   /* message data 1 */
     long data2,   /* message data 2 */
     long data3    /* message data 3 */
){
    XEvent ev;
  
    bzero(&ev, sizeof(ev));
    ev.xclient.type = ClientMessage;
    ev.xclient.window = win;
    ev.xclient.message_type = XInternAtom (dpy, "_NET_SYSTEM_TRAY_OPCODE", False );
    ev.xclient.format = 32;
    ev.xclient.data.l[0] = CurrentTime;
    ev.xclient.data.l[1] = message;
    ev.xclient.data.l[2] = data1;
    ev.xclient.data.l[3] = data2;
    ev.xclient.data.l[4] = data3;

//    trap_errors();
    XSendEvent(dpy, win, False, NoEventMask, &ev);
    XSync(dpy, False);
/*
    if (untrap_errors()) {
    }
*/
}

static void gui_tray_draw(gui_t *gui, winlist_t *win)
{
    xccore_t *xccore = (xccore_t *)win->data;
    IC *ic = xccore->ic;
    if (win->winmode & WMODE_EXIT || !tray_init)
	return;

    inp_state_t inp_state = (ic) ? ic->imc->inp_state : 0;
    char **disp_icon = (inp_state & IM_CINPUT) ? icon_ch : icon_en;
    if (disp_icon == icon)
	return;

    XWindowAttributes attr;
    XGetWindowAttributes(gui->display, win->window, &attr);
    win->width = attr.width;
    win->height = attr.height;
    int x = (attr.width - 20) / 2;
    int y = (attr.height - 20) / 2;

    gui_Draw_Image(win, 1, 1, disp_icon);
    gui_Draw_3D_Box(win, 0, 0, win->width-1, win->height-1, NO_COLOR, 1);

}

/*
    建立 Tray 
*/
winlist_t *gui_tray_init(gui_t *gui, xccore_t *xccore)
{
    winlist_t *win = gui_new_win();
    win->width  = 24;
    win->height = 24;
    win->pos_x = 0;
    win->pos_y = 0;
    win->window = XCreateSimpleWindow(gui->display, gui->root,
		win->pos_x, win->pos_y, win->width, win->height, 1,
		gui_Color(WINBOX_COLOR), gui_Color(BORDER_COLOR));
    win->draw = XftDrawCreate(gui->display, win->window,
		DefaultVisual(gui->display, gui->screen),
		gui->colormap);

    win->data = (void *)xccore;
    win->win_draw_func = gui_tray_draw;
    win->win_mouse_func = NULL;
    win->win_attrib_func = NULL;
    win->win_destroy_func = NULL;
#if 1
    oxim_rc_t *xrc = &(xccore->oxim_rc);
    XSizeHints size_hints;
    XWMHints wm_hints;
    XClassHint class_hints;

    size_hints.flags =
        USPosition | USSize | PMinSize | PResizeInc | PMaxSize | PWinGravity;
    size_hints.x = win->pos_x;
    size_hints.y = win->pos_y;
    size_hints.width = win->width;
    size_hints.height = win->height;
    size_hints.min_width = win->width;
    size_hints.max_width = win->width;
    size_hints.min_height = win->height;
    size_hints.max_height = win->height;
    size_hints.width_inc = win->width;
    size_hints.height_inc = win->height;

    wm_hints.flags = InputHint | StateHint;
    wm_hints.input = False;
    wm_hints.initial_state = NormalState;

    class_hints.res_name = "oxim";
    class_hints.res_class = "oxim";
    XSetWMProperties(gui->display, win->window, NULL,
                NULL, xrc->argv, xrc->argc, &size_hints,
                &wm_hints, &class_hints);
#endif

    XSetWindowAttributes win_attr;
    win_attr.override_redirect = True;
    XChangeWindowAttributes(gui->display, win->window,
				CWOverrideRedirect, &win_attr);

    XSelectInput(gui->display, win->window,
			(StructureNotifyMask | ExposureMask));
    return win;
}

int check_tray_manager(gui_t *gui)
{
    if (tray_init)
	return True;

    char tray_mgr[32];
    sprintf(tray_mgr, "_NET_SYSTEM_TRAY_S%d", gui->screen);
    Atom sys_tray = XInternAtom(gui->display, tray_mgr, False);
    if (sys_tray == None)
    {
	fprintf(stderr, "Can not find system tray!!\n");
	return False;
    }

    XGrabServer(gui->display);
    Window manager_window = XGetSelectionOwner(gui->display, sys_tray);
    if ( manager_window != None )
    {
	XSelectInput(gui->display, manager_window, StructureNotifyMask);
	send_message(gui->display, manager_window, SYSTEM_TRAY_REQUEST_DOCK, gui->tray_win->window, 0, 0);
	tray_init = True;
    }
    else
    {
	fprintf(stderr, "Can not find tray manager window!!\n");
    }
    XUngrabServer(gui->display);
    XFlush(gui->display);

    return tray_init;
}
